﻿using Mapbox.Unity.Location;
using Mapbox.Utils;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class DataPath : MonoBehaviour
{

  [Serializable]
  public struct DataPointCSV
  {
    public string id;
    public float speed;
    public double lat;
    public double lon;
    public int direction;
    public long timestamp;

    public DataPoint ToDataPoint()
    {
      DataPoint dp;
      dp.id_hash = id.GetHashCode();
      dp.speed = speed;
      dp.lat = (float)lat;
      dp.lon = (float)lon;
      dp.direction = direction;
      dp.timestamp = timestamp;
      return dp;
    }
  }

  [Serializable]
  public struct DataPoint
  {
    public int id_hash;
    public float speed;
    public float lat;
    public float lon;
    public int direction;
    public long timestamp;
  }

  public int id_hash;
  public List<DataPoint> points = new List<DataPoint>();
  public Vector3[] positions;
  private Color32 color;

  public static Color32 ConvertAndroidColor(int aCol)
  {
    Color32 c = new Color32();
    c.b = (byte)((aCol) & 0xFF);
    c.g = (byte)((aCol >> 8) & 0xFF);
    c.r = (byte)((aCol >> 16) & 0xFF);
    c.a = 0xFF;
    return c;
  }

  public void Setup()
  {
    this.color = ConvertAndroidColor(this.id_hash);

    //points.Sort(System.Comparison<DataPoint>.cre)
    points.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => isign(l.timestamp - r.timestamp)));


    CalcWorldLines();
    LocationProviderFactory.Instance.mapManager.OnUpdated += () =>
    {
      CalcWorldLines();
    };
  }

  private int isign(long s)
  {
    if(s < 0) return -1;
    if(s > 0) return 1;
    return 0;
  }

  private void CalcWorldLines()
  {
    Mesh mesh;

    var lr = this.GetComponent<LineRenderer>();
    Vector3[] positions = new Vector3[points.Count];

    var map = LocationProviderFactory.Instance.mapManager;
    //Vector3 prevPosition = map.GeoToWorldPosition(new Vector2d(points[0].lat, points[0].lon));
    for (int i = 0; i < points.Count; ++i)
    {
      Vector3 position = map.GeoToWorldPosition(new Vector2d(points[i].lat, points[i].lon));
      //DEBUG
      //position.y = 1;
      positions[i] = position;
    }

    //lr.SetColors()
    this.positions = positions;
    lr.positionCount = positions.Length;
    lr.SetPositions(positions);

    lr.material.color = ConvertAndroidColor(id_hash);

    Gradient gradient;

    gradient = new Gradient();

    // Populate the color keys at the relative time 0 and 1 (0 and 100%)
    var colorKey = new GradientColorKey[2];
    var alphaKey = new GradientAlphaKey[2];

    colorKey[0].color = Color.black;
    colorKey[0].time = 0.0f;
    colorKey[1].color = Color.white;
    colorKey[1].time = 1.0f;

    // Populate the alpha  keys at relative time 0 and 1  (0 and 100%)
    alphaKey[0].alpha = 1.0f;
    alphaKey[0].time = 0.0f;
    alphaKey[1].alpha = 1.0f;
    alphaKey[1].time = 1.0f;

    gradient.SetKeys(colorKey, alphaKey);

    lr.colorGradient = gradient;


    //DEBUG
    //lr.enabled = false;

    this.transform.position = positions[0];
  }

  void LateUpdate()
  {
    if(positions == null || positions.Length == 0) return;
    

    Vector3 prevPosition = positions[0];
    for (int i = 1; i < positions.Length; ++i)
    {
      Vector3 position = positions[i];
      //Debug.DrawLine(prevPosition + Vector3.up, position + Vector3.up, color, 0, true);;
      prevPosition = position;
    }

  }

  private void OnDrawGizmos()
  {
    if(positions == null || positions.Length == 0) return;

    int i = ((int)Time.time * 2) % positions.Length; 

    Gizmos.color = this.color;
    Gizmos.DrawSphere(positions[i], 1);
  }

}
